@@ -1,5 +1,5 @@
 //========================================================================
-// GLFW 3.0 X11 - www.glfw.org
+// GLFW 3.1 X11 - www.glfw.org
 //------------------------------------------------------------------------
 // Copyright (c) 2002-2006 Marcus Geelnard
 // Copyright (c) 2006-2010 Camilla Berglund <elmindreda@elmindreda.org>
@@ -32,46 +32,61 @@
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
+#include <stdio.h>
+#include <locale.h>
 
 
 // Translate an X11 key code to a GLFW key code.
 //
-static int translateKey(int keyCode)
+static int translateKeyCode(int scancode)
 {
     int keySym;
 
-    // Valid key code range is  [8,255], according to the XLib manual
-    if (keyCode < 8 || keyCode > 255)
+    // Valid key code range is  [8,255], according to the Xlib manual
+    if (scancode < 8 || scancode > 255)
         return GLFW_KEY_UNKNOWN;
 
-    // Try secondary keysym, for numeric keypad keys
-    // Note: This way we always force "NumLock = ON", which is intentional
-    // since the returned key code should correspond to a physical
-    // location.
-    keySym = XkbKeycodeToKeysym(_glfw.x11.display, keyCode, 0, 1);
-    switch (keySym)
+    if (_glfw.x11.xkb.available)
     {
-        case XK_KP_0:           return GLFW_KEY_KP_0;
-        case XK_KP_1:           return GLFW_KEY_KP_1;
-        case XK_KP_2:           return GLFW_KEY_KP_2;
-        case XK_KP_3:           return GLFW_KEY_KP_3;
-        case XK_KP_4:           return GLFW_KEY_KP_4;
-        case XK_KP_5:           return GLFW_KEY_KP_5;
-        case XK_KP_6:           return GLFW_KEY_KP_6;
-        case XK_KP_7:           return GLFW_KEY_KP_7;
-        case XK_KP_8:           return GLFW_KEY_KP_8;
-        case XK_KP_9:           return GLFW_KEY_KP_9;
-        case XK_KP_Separator:
-        case XK_KP_Decimal:     return GLFW_KEY_KP_DECIMAL;
-        case XK_KP_Equal:       return GLFW_KEY_KP_EQUAL;
-        case XK_KP_Enter:       return GLFW_KEY_KP_ENTER;
-        default:                break;
+        // Try secondary keysym, for numeric keypad keys
+        // Note: This way we always force "NumLock = ON", which is intentional
+        // since the returned key code should correspond to a physical
+        // location.
+        keySym = XkbKeycodeToKeysym(_glfw.x11.display, scancode, 0, 1);
+        switch (keySym)
+        {
+            case XK_KP_0:           return GLFW_KEY_KP_0;
+            case XK_KP_1:           return GLFW_KEY_KP_1;
+            case XK_KP_2:           return GLFW_KEY_KP_2;
+            case XK_KP_3:           return GLFW_KEY_KP_3;
+            case XK_KP_4:           return GLFW_KEY_KP_4;
+            case XK_KP_5:           return GLFW_KEY_KP_5;
+            case XK_KP_6:           return GLFW_KEY_KP_6;
+            case XK_KP_7:           return GLFW_KEY_KP_7;
+            case XK_KP_8:           return GLFW_KEY_KP_8;
+            case XK_KP_9:           return GLFW_KEY_KP_9;
+            case XK_KP_Separator:
+            case XK_KP_Decimal:     return GLFW_KEY_KP_DECIMAL;
+            case XK_KP_Equal:       return GLFW_KEY_KP_EQUAL;
+            case XK_KP_Enter:       return GLFW_KEY_KP_ENTER;
+            default:                break;
+        }
+
+        // Now try primary keysym for function keys (non-printable keys). These
+        // should not be layout dependent (i.e. US layout and international
+        // layouts should give the same result).
+        keySym = XkbKeycodeToKeysym(_glfw.x11.display, scancode, 0, 0);
+    }
+    else
+    {
+        int dummy;
+        KeySym* keySyms;
+
+        keySyms = XGetKeyboardMapping(_glfw.x11.display, scancode, 1, &dummy);
+        keySym = keySyms[0];
+        XFree(keySyms);
     }
 
-    // Now try pimary keysym for function keys (non-printable keys). These
-    // should not be layout dependent (i.e. US layout and international
-    // layouts should give the same result).
-    keySym = XkbKeycodeToKeysym(_glfw.x11.display, keyCode, 0, 0);
     switch (keySym)
     {
         case XK_Escape:         return GLFW_KEY_ESCAPE;
@@ -212,106 +227,124 @@ static int translateKey(int keyCode)
     return GLFW_KEY_UNKNOWN;
 }
 
-// Update the key code LUT
+// Create key code translation tables
 //
-static void updateKeyCodeLUT(void)
+static void createKeyTables(void)
 {
-    int i, keyCode, keyCodeGLFW;
-    char name[XkbKeyNameLength + 1];
-    XkbDescPtr descr;
+    int scancode, key;
 
-    // Clear the LUT
-    for (keyCode = 0;  keyCode < 256;  keyCode++)
-        _glfw.x11.keyCodeLUT[keyCode] = GLFW_KEY_UNKNOWN;
+    memset(_glfw.x11.publicKeys, -1, sizeof(_glfw.x11.publicKeys));
 
-    // Use XKB to determine physical key locations independently of the current
-    // keyboard layout
-
-    // Get keyboard description
-    descr = XkbGetKeyboard(_glfw.x11.display,
-                            XkbAllComponentsMask,
-                            XkbUseCoreKbd);
-
-    // Find the X11 key code -> GLFW key code mapping
-    for (keyCode = descr->min_key_code; keyCode <= descr->max_key_code; ++keyCode)
+    if (_glfw.x11.xkb.available)
     {
-        // Get the key name
-        for (i = 0;  i < XkbKeyNameLength;  i++)
-            name[i] = descr->names->keys[keyCode].name[i];
+        // Use XKB to determine physical key locations independently of the current
+        // keyboard layout
 
-        name[XkbKeyNameLength] = 0;
+        char name[XkbKeyNameLength + 1];
+        XkbDescPtr desc = XkbGetMap(_glfw.x11.display, 0, XkbUseCoreKbd);
+        XkbGetNames(_glfw.x11.display, XkbKeyNamesMask, desc);
 
-        // Map the key name to a GLFW key code. Note: We only map printable
-        // keys here, and we use the US keyboard layout. The rest of the
-        // keys (function keys) are mapped using traditional KeySym
-        // translations.
-        if (strcmp(name, "TLDE") == 0) keyCodeGLFW = GLFW_KEY_GRAVE_ACCENT;
-        else if (strcmp(name, "AE01") == 0) keyCodeGLFW = GLFW_KEY_1;
-        else if (strcmp(name, "AE02") == 0) keyCodeGLFW = GLFW_KEY_2;
-        else if (strcmp(name, "AE03") == 0) keyCodeGLFW = GLFW_KEY_3;
-        else if (strcmp(name, "AE04") == 0) keyCodeGLFW = GLFW_KEY_4;
-        else if (strcmp(name, "AE05") == 0) keyCodeGLFW = GLFW_KEY_5;
-        else if (strcmp(name, "AE06") == 0) keyCodeGLFW = GLFW_KEY_6;
-        else if (strcmp(name, "AE07") == 0) keyCodeGLFW = GLFW_KEY_7;
-        else if (strcmp(name, "AE08") == 0) keyCodeGLFW = GLFW_KEY_8;
-        else if (strcmp(name, "AE09") == 0) keyCodeGLFW = GLFW_KEY_9;
-        else if (strcmp(name, "AE10") == 0) keyCodeGLFW = GLFW_KEY_0;
-        else if (strcmp(name, "AE11") == 0) keyCodeGLFW = GLFW_KEY_MINUS;
-        else if (strcmp(name, "AE12") == 0) keyCodeGLFW = GLFW_KEY_EQUAL;
-        else if (strcmp(name, "AD01") == 0) keyCodeGLFW = GLFW_KEY_Q;
-        else if (strcmp(name, "AD02") == 0) keyCodeGLFW = GLFW_KEY_W;
-        else if (strcmp(name, "AD03") == 0) keyCodeGLFW = GLFW_KEY_E;
-        else if (strcmp(name, "AD04") == 0) keyCodeGLFW = GLFW_KEY_R;
-        else if (strcmp(name, "AD05") == 0) keyCodeGLFW = GLFW_KEY_T;
-        else if (strcmp(name, "AD06") == 0) keyCodeGLFW = GLFW_KEY_Y;
-        else if (strcmp(name, "AD07") == 0) keyCodeGLFW = GLFW_KEY_U;
-        else if (strcmp(name, "AD08") == 0) keyCodeGLFW = GLFW_KEY_I;
-        else if (strcmp(name, "AD09") == 0) keyCodeGLFW = GLFW_KEY_O;
-        else if (strcmp(name, "AD10") == 0) keyCodeGLFW = GLFW_KEY_P;
-        else if (strcmp(name, "AD11") == 0) keyCodeGLFW = GLFW_KEY_LEFT_BRACKET;
-        else if (strcmp(name, "AD12") == 0) keyCodeGLFW = GLFW_KEY_RIGHT_BRACKET;
-        else if (strcmp(name, "AC01") == 0) keyCodeGLFW = GLFW_KEY_A;
-        else if (strcmp(name, "AC02") == 0) keyCodeGLFW = GLFW_KEY_S;
-        else if (strcmp(name, "AC03") == 0) keyCodeGLFW = GLFW_KEY_D;
-        else if (strcmp(name, "AC04") == 0) keyCodeGLFW = GLFW_KEY_F;
-        else if (strcmp(name, "AC05") == 0) keyCodeGLFW = GLFW_KEY_G;
-        else if (strcmp(name, "AC06") == 0) keyCodeGLFW = GLFW_KEY_H;
-        else if (strcmp(name, "AC07") == 0) keyCodeGLFW = GLFW_KEY_J;
-        else if (strcmp(name, "AC08") == 0) keyCodeGLFW = GLFW_KEY_K;
-        else if (strcmp(name, "AC09") == 0) keyCodeGLFW = GLFW_KEY_L;
-        else if (strcmp(name, "AC10") == 0) keyCodeGLFW = GLFW_KEY_SEMICOLON;
-        else if (strcmp(name, "AC11") == 0) keyCodeGLFW = GLFW_KEY_APOSTROPHE;
-        else if (strcmp(name, "AB01") == 0) keyCodeGLFW = GLFW_KEY_Z;
-        else if (strcmp(name, "AB02") == 0) keyCodeGLFW = GLFW_KEY_X;
-        else if (strcmp(name, "AB03") == 0) keyCodeGLFW = GLFW_KEY_C;
-        else if (strcmp(name, "AB04") == 0) keyCodeGLFW = GLFW_KEY_V;
-        else if (strcmp(name, "AB05") == 0) keyCodeGLFW = GLFW_KEY_B;
-        else if (strcmp(name, "AB06") == 0) keyCodeGLFW = GLFW_KEY_N;
-        else if (strcmp(name, "AB07") == 0) keyCodeGLFW = GLFW_KEY_M;
-        else if (strcmp(name, "AB08") == 0) keyCodeGLFW = GLFW_KEY_COMMA;
-        else if (strcmp(name, "AB09") == 0) keyCodeGLFW = GLFW_KEY_PERIOD;
-        else if (strcmp(name, "AB10") == 0) keyCodeGLFW = GLFW_KEY_SLASH;
-        else if (strcmp(name, "BKSL") == 0) keyCodeGLFW = GLFW_KEY_BACKSLASH;
-        else if (strcmp(name, "LSGT") == 0) keyCodeGLFW = GLFW_KEY_WORLD_1;
-        else keyCodeGLFW = GLFW_KEY_UNKNOWN;
+        // Find the X11 key code -> GLFW key code mapping
+        for (scancode = desc->min_key_code;  scancode <= desc->max_key_code;  scancode++)
+        {
+            memcpy(name, desc->names->keys[scancode].name, XkbKeyNameLength);
+            name[XkbKeyNameLength] = '\0';
 
-        // Update the key code LUT
-        if ((keyCode >= 0) && (keyCode < 256))
-            _glfw.x11.keyCodeLUT[keyCode] = keyCodeGLFW;
+            // Map the key name to a GLFW key code. Note: We only map printable
+            // keys here, and we use the US keyboard layout. The rest of the
+            // keys (function keys) are mapped using traditional KeySym
+            // translations.
+            if (strcmp(name, "TLDE") == 0) key = GLFW_KEY_GRAVE_ACCENT;
+            else if (strcmp(name, "AE01") == 0) key = GLFW_KEY_1;
+            else if (strcmp(name, "AE02") == 0) key = GLFW_KEY_2;
+            else if (strcmp(name, "AE03") == 0) key = GLFW_KEY_3;
+            else if (strcmp(name, "AE04") == 0) key = GLFW_KEY_4;
+            else if (strcmp(name, "AE05") == 0) key = GLFW_KEY_5;
+            else if (strcmp(name, "AE06") == 0) key = GLFW_KEY_6;
+            else if (strcmp(name, "AE07") == 0) key = GLFW_KEY_7;
+            else if (strcmp(name, "AE08") == 0) key = GLFW_KEY_8;
+            else if (strcmp(name, "AE09") == 0) key = GLFW_KEY_9;
+            else if (strcmp(name, "AE10") == 0) key = GLFW_KEY_0;
+            else if (strcmp(name, "AE11") == 0) key = GLFW_KEY_MINUS;
+            else if (strcmp(name, "AE12") == 0) key = GLFW_KEY_EQUAL;
+            else if (strcmp(name, "AD01") == 0) key = GLFW_KEY_Q;
+            else if (strcmp(name, "AD02") == 0) key = GLFW_KEY_W;
+            else if (strcmp(name, "AD03") == 0) key = GLFW_KEY_E;
+            else if (strcmp(name, "AD04") == 0) key = GLFW_KEY_R;
+            else if (strcmp(name, "AD05") == 0) key = GLFW_KEY_T;
+            else if (strcmp(name, "AD06") == 0) key = GLFW_KEY_Y;
+            else if (strcmp(name, "AD07") == 0) key = GLFW_KEY_U;
+            else if (strcmp(name, "AD08") == 0) key = GLFW_KEY_I;
+            else if (strcmp(name, "AD09") == 0) key = GLFW_KEY_O;
+            else if (strcmp(name, "AD10") == 0) key = GLFW_KEY_P;
+            else if (strcmp(name, "AD11") == 0) key = GLFW_KEY_LEFT_BRACKET;
+            else if (strcmp(name, "AD12") == 0) key = GLFW_KEY_RIGHT_BRACKET;
+            else if (strcmp(name, "AC01") == 0) key = GLFW_KEY_A;
+            else if (strcmp(name, "AC02") == 0) key = GLFW_KEY_S;
+            else if (strcmp(name, "AC03") == 0) key = GLFW_KEY_D;
+            else if (strcmp(name, "AC04") == 0) key = GLFW_KEY_F;
+            else if (strcmp(name, "AC05") == 0) key = GLFW_KEY_G;
+            else if (strcmp(name, "AC06") == 0) key = GLFW_KEY_H;
+            else if (strcmp(name, "AC07") == 0) key = GLFW_KEY_J;
+            else if (strcmp(name, "AC08") == 0) key = GLFW_KEY_K;
+            else if (strcmp(name, "AC09") == 0) key = GLFW_KEY_L;
+            else if (strcmp(name, "AC10") == 0) key = GLFW_KEY_SEMICOLON;
+            else if (strcmp(name, "AC11") == 0) key = GLFW_KEY_APOSTROPHE;
+            else if (strcmp(name, "AB01") == 0) key = GLFW_KEY_Z;
+            else if (strcmp(name, "AB02") == 0) key = GLFW_KEY_X;
+            else if (strcmp(name, "AB03") == 0) key = GLFW_KEY_C;
+            else if (strcmp(name, "AB04") == 0) key = GLFW_KEY_V;
+            else if (strcmp(name, "AB05") == 0) key = GLFW_KEY_B;
+            else if (strcmp(name, "AB06") == 0) key = GLFW_KEY_N;
+            else if (strcmp(name, "AB07") == 0) key = GLFW_KEY_M;
+            else if (strcmp(name, "AB08") == 0) key = GLFW_KEY_COMMA;
+            else if (strcmp(name, "AB09") == 0) key = GLFW_KEY_PERIOD;
+            else if (strcmp(name, "AB10") == 0) key = GLFW_KEY_SLASH;
+            else if (strcmp(name, "BKSL") == 0) key = GLFW_KEY_BACKSLASH;
+            else if (strcmp(name, "LSGT") == 0) key = GLFW_KEY_WORLD_1;
+            else key = GLFW_KEY_UNKNOWN;
+
+            if ((scancode >= 0) && (scancode < 256))
+                _glfw.x11.publicKeys[scancode] = key;
+        }
+
+        XkbFreeNames(desc, XkbKeyNamesMask, True);
+        XkbFreeClientMap(desc, 0, True);
     }
 
-    // Free the keyboard description
-    XkbFreeKeyboard(descr, 0, True);
-
     // Translate the un-translated key codes using traditional X11 KeySym
     // lookups
-    for (keyCode = 0;  keyCode < 256;  keyCode++)
+    for (scancode = 0;  scancode < 256;  scancode++)
     {
-        if (_glfw.x11.keyCodeLUT[keyCode] < 0)
-            _glfw.x11.keyCodeLUT[keyCode] = translateKey(keyCode);
+        if (_glfw.x11.publicKeys[scancode] < 0)
+            _glfw.x11.publicKeys[scancode] = translateKeyCode(scancode);
     }
 }
 
+// Check whether the IM has a usable style
+//
+static GLboolean hasUsableInputMethodStyle(void)
+{
+    unsigned int i;
+    GLboolean found = GL_FALSE;
+    XIMStyles* styles = NULL;
+
+    if (XGetIMValues(_glfw.x11.im, XNQueryInputStyle, &styles, NULL) != NULL)
+        return GL_FALSE;
+
+    for (i = 0;  i < styles->count_styles;  i++)
+    {
+        if (styles->supported_styles[i] == (XIMPreeditNothing | XIMStatusNothing))
+        {
+            found = GL_TRUE;
+            break;
+        }
+    }
+
+    XFree(styles);
+    return found;
+}
+
 // Check whether the specified atom is supported
 //
 static Atom getSupportedAtom(Atom* supportedAtoms,
@@ -354,10 +387,13 @@ static void detectEWMH(void)
                                XA_WINDOW,
                                (unsigned char**) &windowFromRoot) != 1)
     {
-        XFree(windowFromRoot);
+        if (windowFromRoot)
+            XFree(windowFromRoot);
         return;
     }
 
+    _glfwGrabXErrorHandler();
+
     // It should be the ID of a child window (of the root)
     // Then we look for the same property on the child window
     if (_glfwGetWindowProperty(*windowFromRoot,
@@ -366,10 +402,13 @@ static void detectEWMH(void)
                                (unsigned char**) &windowFromChild) != 1)
     {
         XFree(windowFromRoot);
-        XFree(windowFromChild);
+        if (windowFromChild)
+            XFree(windowFromChild);
         return;
     }
 
+    _glfwReleaseXErrorHandler();
+
     // It should be the ID of that same child window
     if (*windowFromRoot != *windowFromChild)
     {
@@ -394,11 +433,14 @@ static void detectEWMH(void)
                                        (unsigned char**) &supportedAtoms);
 
     // See which of the atoms we support that are supported by the WM
-
     _glfw.x11.NET_WM_STATE =
         getSupportedAtom(supportedAtoms, atomCount, "_NET_WM_STATE");
+    _glfw.x11.NET_WM_STATE_ABOVE =
+        getSupportedAtom(supportedAtoms, atomCount, "_NET_WM_STATE_ABOVE");
     _glfw.x11.NET_WM_STATE_FULLSCREEN =
         getSupportedAtom(supportedAtoms, atomCount, "_NET_WM_STATE_FULLSCREEN");
+    _glfw.x11.NET_WM_FULLSCREEN_MONITORS =
+        getSupportedAtom(supportedAtoms, atomCount, "_NET_WM_FULLSCREEN_MONITORS");
     _glfw.x11.NET_WM_NAME =
         getSupportedAtom(supportedAtoms, atomCount, "_NET_WM_NAME");
     _glfw.x11.NET_WM_ICON_NAME =
@@ -409,21 +451,24 @@ static void detectEWMH(void)
         getSupportedAtom(supportedAtoms, atomCount, "_NET_WM_PING");
     _glfw.x11.NET_ACTIVE_WINDOW =
         getSupportedAtom(supportedAtoms, atomCount, "_NET_ACTIVE_WINDOW");
+    _glfw.x11.NET_FRAME_EXTENTS =
+        getSupportedAtom(supportedAtoms, atomCount, "_NET_FRAME_EXTENTS");
+    _glfw.x11.NET_REQUEST_FRAME_EXTENTS =
+        getSupportedAtom(supportedAtoms, atomCount, "_NET_REQUEST_FRAME_EXTENTS");
     _glfw.x11.NET_WM_BYPASS_COMPOSITOR =
         getSupportedAtom(supportedAtoms, atomCount, "_NET_WM_BYPASS_COMPOSITOR");
 
     XFree(supportedAtoms);
-
-    _glfw.x11.hasEWMH = GL_TRUE;
 }
 
 // Initialize X11 display and look for supported X11 extensions
 //
 static GLboolean initExtensions(void)
 {
-    Bool supported;
-
     // Find or create window manager atoms
+    _glfw.x11.WM_PROTOCOLS = XInternAtom(_glfw.x11.display,
+                                         "WM_PROTOCOLS",
+                                         False);
     _glfw.x11.WM_STATE = XInternAtom(_glfw.x11.display, "WM_STATE", False);
     _glfw.x11.WM_DELETE_WINDOW = XInternAtom(_glfw.x11.display,
                                              "WM_DELETE_WINDOW",
@@ -432,11 +477,13 @@ static GLboolean initExtensions(void)
                                            "_MOTIF_WM_HINTS",
                                            False);
 
+#if defined(_GLFW_HAS_XF86VM)
     // Check for XF86VidMode extension
     _glfw.x11.vidmode.available =
         XF86VidModeQueryExtension(_glfw.x11.display,
                                   &_glfw.x11.vidmode.eventBase,
                                   &_glfw.x11.vidmode.errorBase);
+#endif /*_GLFW_HAS_XF86VM*/
 
     // Check for RandR extension
     _glfw.x11.randr.available =
@@ -446,9 +493,11 @@ static GLboolean initExtensions(void)
 
     if (_glfw.x11.randr.available)
     {
+        XRRScreenResources* sr;
+
         if (!XRRQueryVersion(_glfw.x11.display,
-                             &_glfw.x11.randr.versionMajor,
-                             &_glfw.x11.randr.versionMinor))
+                             &_glfw.x11.randr.major,
+                             &_glfw.x11.randr.minor))
         {
             _glfwInputError(GLFW_PLATFORM_ERROR,
                             "X11: Failed to query RandR version");
@@ -456,68 +505,84 @@ static GLboolean initExtensions(void)
         }
 
         // The GLFW RandR path requires at least version 1.3
-        if (_glfw.x11.randr.versionMajor == 1 &&
-            _glfw.x11.randr.versionMinor < 3)
-        {
+        if (_glfw.x11.randr.major == 1 && _glfw.x11.randr.minor < 3)
             _glfw.x11.randr.available = GL_FALSE;
+
+        sr = XRRGetScreenResources(_glfw.x11.display, _glfw.x11.root);
+
+        if (!sr->ncrtc || !XRRGetCrtcGammaSize(_glfw.x11.display, sr->crtcs[0]))
+        {
+            // This is either a headless system or an older Nvidia binary driver
+            // with broken gamma support
+            // Flag it as useless and fall back to Xf86VidMode gamma, if
+            // available
+            _glfwInputError(GLFW_PLATFORM_ERROR,
+                            "X11: RandR gamma ramp support seems broken");
+            _glfw.x11.randr.gammaBroken = GL_TRUE;
         }
+
+        XRRFreeScreenResources(sr);
     }
 
+    if (XineramaQueryExtension(_glfw.x11.display,
+                               &_glfw.x11.xinerama.major,
+                               &_glfw.x11.xinerama.minor))
+    {
+        if (XineramaIsActive(_glfw.x11.display))
+            _glfw.x11.xinerama.available = GL_TRUE;
+    }
+
+#if defined(_GLFW_HAS_XINPUT)
     if (XQueryExtension(_glfw.x11.display,
                         "XInputExtension",
                         &_glfw.x11.xi.majorOpcode,
                         &_glfw.x11.xi.eventBase,
                         &_glfw.x11.xi.errorBase))
     {
-        _glfw.x11.xi.versionMajor = 2;
-        _glfw.x11.xi.versionMinor = 0;
+        _glfw.x11.xi.major = 2;
+        _glfw.x11.xi.minor = 0;
 
         if (XIQueryVersion(_glfw.x11.display,
-                           &_glfw.x11.xi.versionMajor,
-                           &_glfw.x11.xi.versionMinor) != BadRequest)
+                           &_glfw.x11.xi.major,
+                           &_glfw.x11.xi.minor) != BadRequest)
         {
             _glfw.x11.xi.available = GL_TRUE;
         }
     }
+#endif /*_GLFW_HAS_XINPUT*/
 
     // Check if Xkb is supported on this display
-    _glfw.x11.xkb.versionMajor = 1;
-    _glfw.x11.xkb.versionMinor = 0;
-    if (!XkbQueryExtension(_glfw.x11.display,
-                           &_glfw.x11.xkb.majorOpcode,
-                           &_glfw.x11.xkb.eventBase,
-                           &_glfw.x11.xkb.errorBase,
-                           &_glfw.x11.xkb.versionMajor,
-                           &_glfw.x11.xkb.versionMinor))
-    {
-        _glfwInputError(GLFW_PLATFORM_ERROR,
-                        "X11: The keyboard extension is not available");
-        return GL_FALSE;
-    }
+    _glfw.x11.xkb.major = 1;
+    _glfw.x11.xkb.minor = 0;
+    _glfw.x11.xkb.available =
+        XkbQueryExtension(_glfw.x11.display,
+                          &_glfw.x11.xkb.majorOpcode,
+                          &_glfw.x11.xkb.eventBase,
+                          &_glfw.x11.xkb.errorBase,
+                          &_glfw.x11.xkb.major,
+                          &_glfw.x11.xkb.minor);
 
-    if (!XkbSetDetectableAutoRepeat(_glfw.x11.display, True, &supported))
+    if (_glfw.x11.xkb.available)
     {
-        _glfwInputError(GLFW_PLATFORM_ERROR,
-                        "X11: Failed to set detectable key repeat");
-        return GL_FALSE;
-    }
+        Bool supported;
 
-    if (!supported)
-    {
-        _glfwInputError(GLFW_PLATFORM_ERROR,
-                        "X11: Detectable key repeat is not supported");
-        return GL_FALSE;
+        if (XkbSetDetectableAutoRepeat(_glfw.x11.display, True, &supported))
+        {
+            if (supported)
+                _glfw.x11.xkb.detectable = GL_TRUE;
+        }
     }
 
     // Update the key code LUT
     // FIXME: We should listen to XkbMapNotify events to track changes to
     // the keyboard mapping.
-    updateKeyCodeLUT();
+    createKeyTables();
 
     // Detect whether an EWMH-conformant window manager is running
     detectEWMH();
 
     // Find or create string format atoms
+    _glfw.x11.NULL_ = XInternAtom(_glfw.x11.display, "NULL", False);
     _glfw.x11.UTF8_STRING =
         XInternAtom(_glfw.x11.display, "UTF8_STRING", False);
     _glfw.x11.COMPOUND_STRING =
@@ -539,54 +604,30 @@ static GLboolean initExtensions(void)
     _glfw.x11.SAVE_TARGETS =
         XInternAtom(_glfw.x11.display, "SAVE_TARGETS", False);
 
+    // Find Xdnd (drag and drop) atoms, if available
+    _glfw.x11.XdndAware = XInternAtom(_glfw.x11.display, "XdndAware", True);
+    _glfw.x11.XdndEnter = XInternAtom(_glfw.x11.display, "XdndEnter", True);
+    _glfw.x11.XdndPosition = XInternAtom(_glfw.x11.display, "XdndPosition", True);
+    _glfw.x11.XdndStatus = XInternAtom(_glfw.x11.display, "XdndStatus", True);
+    _glfw.x11.XdndActionCopy = XInternAtom(_glfw.x11.display, "XdndActionCopy", True);
+    _glfw.x11.XdndDrop = XInternAtom(_glfw.x11.display, "XdndDrop", True);
+    _glfw.x11.XdndLeave = XInternAtom(_glfw.x11.display, "XdndLeave", True);
+    _glfw.x11.XdndFinished = XInternAtom(_glfw.x11.display, "XdndFinished", True);
+    _glfw.x11.XdndSelection = XInternAtom(_glfw.x11.display, "XdndSelection", True);
+
     return GL_TRUE;
 }
 
-// Create a blank cursor (for locked mouse mode)
+// Create a blank cursor for hidden and disabled cursor modes
 //
 static Cursor createNULLCursor(void)
 {
-    Pixmap cursormask;
-    XGCValues xgc;
-    GC gc;
-    XColor col;
-    Cursor cursor;
+    unsigned char pixels[16 * 16 * 4];
+    GLFWimage image = { 16, 16, pixels };
 
-    _glfwGrabXErrorHandler();
+    memset(pixels, 0, sizeof(pixels));
 
-    cursormask = XCreatePixmap(_glfw.x11.display, _glfw.x11.root, 1, 1, 1);
-    xgc.function = GXclear;
-    gc = XCreateGC(_glfw.x11.display, cursormask, GCFunction, &xgc);
-    XFillRectangle(_glfw.x11.display, cursormask, gc, 0, 0, 1, 1);
-    col.pixel = 0;
-    col.red = 0;
-    col.flags = 4;
-    cursor = XCreatePixmapCursor(_glfw.x11.display,
-                                 cursormask, cursormask,
-                                 &col, &col, 0, 0);
-    XFreePixmap(_glfw.x11.display, cursormask);
-    XFreeGC(_glfw.x11.display, gc);
-
-    _glfwReleaseXErrorHandler();
-
-    if (cursor == None)
-    {
-        _glfwInputXError(GLFW_PLATFORM_ERROR,
-                         "X11: Failed to create null cursor");
-    }
-
-    return cursor;
-}
-
-// Terminate X11 display
-//
-static void terminateDisplay(void)
-{
-    if (_glfw.x11.display)
-    {
-        XCloseDisplay(_glfw.x11.display);
-        _glfw.x11.display = NULL;
-    }
+    return _glfwCreateCursor(&image, 0, 0);
 }
 
 // X error handler
@@ -602,7 +643,7 @@ static int errorHandler(Display *display, XErrorEvent* event)
 //////                       GLFW internal API                      //////
 //////////////////////////////////////////////////////////////////////////
 
-// Install the X error handler
+// Sets the X error handler callback
 //
 void _glfwGrabXErrorHandler(void)
 {
@@ -610,7 +651,7 @@ void _glfwGrabXErrorHandler(void)
     XSetErrorHandler(errorHandler);
 }
 
-// Remove the X error handler
+// Clears the X error handler callback
 //
 void _glfwReleaseXErrorHandler(void)
 {
@@ -619,7 +660,7 @@ void _glfwReleaseXErrorHandler(void)
     XSetErrorHandler(NULL);
 }
 
-// Report X error
+// Reports the specified error, appending information about the last X error
 //
 void _glfwInputXError(int error, const char* message)
 {
@@ -630,6 +671,37 @@ void _glfwInputXError(int error, const char* message)
     _glfwInputError(error, "%s: %s", message, buffer);
 }
 
+// Creates a native cursor object from the specified image and hotspot
+//
+Cursor _glfwCreateCursor(const GLFWimage* image, int xhot, int yhot)
+{
+    int i;
+    Cursor cursor;
+
+    XcursorImage* native = XcursorImageCreate(image->width, image->height);
+    if (native == NULL)
+        return None;
+
+    native->xhot = xhot;
+    native->yhot = yhot;
+
+    unsigned char* source = (unsigned char*) image->pixels;
+    XcursorPixel* target = native->pixels;
+
+    for (i = 0;  i < image->width * image->height;  i++, target++, source += 4)
+    {
+        *target = (source[3] << 24) |
+                  (source[0] << 16) |
+                  (source[1] <<  8) |
+                   source[2];
+    }
+
+    cursor = XcursorImageLoadCursor(_glfw.x11.display, native);
+    XcursorImageDestroy(native);
+
+    return cursor;
+}
+
 
 //////////////////////////////////////////////////////////////////////////
 //////                       GLFW platform API                      //////
@@ -637,12 +709,30 @@ void _glfwInputXError(int error, const char* message)
 
 int _glfwPlatformInit(void)
 {
+#if !defined(X_HAVE_UTF8_STRING)
+    // HACK: If the current locale is C, apply the environment's locale
+    //       This is done because the C locale breaks wide character input
+    if (strcmp(setlocale(LC_CTYPE, NULL), "C") == 0)
+        setlocale(LC_CTYPE, "");
+#endif
+
     XInitThreads();
 
     _glfw.x11.display = XOpenDisplay(NULL);
     if (!_glfw.x11.display)
     {
-        _glfwInputError(GLFW_API_UNAVAILABLE, "X11: Failed to open X display");
+        const char* display = getenv("DISPLAY");
+        if (display)
+        {
+            _glfwInputError(GLFW_PLATFORM_ERROR,
+                            "X11: Failed to open display %s", display);
+        }
+        else
+        {
+            _glfwInputError(GLFW_PLATFORM_ERROR,
+                            "X11: The DISPLAY environment variable is missing");
+        }
+
         return GL_FALSE;
     }
 
@@ -655,12 +745,28 @@ int _glfwPlatformInit(void)
 
     _glfw.x11.cursor = createNULLCursor();
 
+    if (XSupportsLocale())
+    {
+        XSetLocaleModifiers("");
+
+        _glfw.x11.im = XOpenIM(_glfw.x11.display, 0, NULL, NULL);
+        if (_glfw.x11.im)
+        {
+            if (!hasUsableInputMethodStyle())
+            {
+                XCloseIM(_glfw.x11.im);
+                _glfw.x11.im = NULL;
+            }
+        }
+    }
+
     if (!_glfwInitContextAPI())
         return GL_FALSE;
 
+    if (!_glfwInitJoysticks())
+        return GL_FALSE;
+
     _glfwInitTimer();
-    _glfwInitJoysticks();
-    _glfwInitGammaRamp();
 
     return GL_TRUE;
 }
@@ -673,41 +779,52 @@ void _glfwPlatformTerminate(void)
         _glfw.x11.cursor = (Cursor) 0;
     }
 
-    free(_glfw.x11.selection.string);
+    free(_glfw.x11.clipboardString);
+
+    if (_glfw.x11.im)
+    {
+        XCloseIM(_glfw.x11.im);
+        _glfw.x11.im = NULL;
+    }
 
     _glfwTerminateJoysticks();
+
+    if (_glfw.x11.display)
+    {
+        XCloseDisplay(_glfw.x11.display);
+        _glfw.x11.display = NULL;
+    }
+
+    // NOTE: This needs to be done after XCloseDisplay, as libGL registers
+    //       internal cleanup callbacks in libX11
     _glfwTerminateContextAPI();
-    terminateDisplay();
 }
 
 const char* _glfwPlatformGetVersionString(void)
 {
-    const char* version = _GLFW_VERSION_NUMBER " X11"
+    return _GLFW_VERSION_NUMBER " X11"
 #if defined(_GLFW_GLX)
         " GLX"
 #elif defined(_GLFW_EGL)
         " EGL"
 #endif
-#if defined(_GLFW_HAS_GLXGETPROCADDRESS)
-        " glXGetProcAddress"
-#elif defined(_GLFW_HAS_GLXGETPROCADDRESSARB)
-        " glXGetProcAddressARB"
-#elif defined(_GLFW_HAS_GLXGETPROCADDRESSEXT)
-        " glXGetProcAddressEXT"
-#elif defined(_GLFW_DLOPEN_LIBGL)
-        " dlsym(libGL)"
-#endif
 #if defined(_POSIX_TIMERS) && defined(_POSIX_MONOTONIC_CLOCK)
         " clock_gettime"
+#else
+        " gettimeofday"
 #endif
 #if defined(__linux__)
         " /dev/js"
 #endif
+#if defined(_GLFW_HAS_XINPUT)
+        " XI"
+#endif
+#if defined(_GLFW_HAS_XF86VM)
+        " Xf86vm"
+#endif
 #if defined(_GLFW_BUILD_DLL)
         " shared"
 #endif
         ;
-
-    return version;
 }
 
