@@ -110,7 +110,7 @@ static void draw_joystick(Joystick* j, int x, int y, int width, int height)
 
 static void draw_joysticks(GLFWwindow* window)
 {
-    int i, width, height;
+    int i, width, height, offset = 0;
 
     glfwGetFramebufferSize(window, &width, &height);
 
@@ -126,8 +126,9 @@ static void draw_joysticks(GLFWwindow* window)
         if (j->present)
         {
             draw_joystick(j,
-                          0, i * height / joystick_count,
+                          0, offset * height / joystick_count,
                           width, height / joystick_count);
+            offset++;
         }
     }
 }
